//  QUWidget.h version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//
// Gaspar Sinai <gsinai@gol.com>
//

#ifndef __quwidget_h__
#define __quwidget_h__

#include <UCommon.h>
#include <AString.h>
#include <qwidget.h>
#include <qframe.h>
#include <qscrollbar.h>
#include <qcolor.h>

#include <UXInput.h>
#include <UWidget.h>
#include <UString.h>
#include <UFontMap.h>
#include <UFont.h>

class QUWidgetBase : public QWidget, public UWidget
{
	Q_OBJECT
public:
				QUWidgetBase (QWidget* parent, 
					const char* name);
				~QUWidgetBase();
	UXInput*		uEventHandler[UXInput::U_MAX_EVENT];
signals:
	void			setEVScroll (int value, int step, int page, int max);
	void			setEHScroll (int value, int step, int page, int max);
protected:
	// override methods here.
	virtual bool		x11Event (XEvent *);
	virtual void		uClipboardPut (const unsigned char *, const int len);
	virtual unsigned char*	uClipboardGet (int* len);
	virtual void		paintEvent (QPaintEvent * event);
	virtual void		resizeEvent (QResizeEvent* event);
	virtual void		keyPressEvent (QKeyEvent *event);
	virtual void		mousePressEvent (QMouseEvent *event);
	virtual void		mouseReleaseEvent (QMouseEvent *event);
	virtual void		mouseMoveEvent (QMouseEvent *event);
	virtual void		timerEvent (QTimerEvent *event);
	virtual void		backgroundColorChange (const QColor &old);
	void			focusInEvent (QFocusEvent *event);
	void			focusOutEvent (QFocusEvent *event);
	void			uSetVScroll (int value, int step, int page, int max);
	void			uSetHScroll (int vlaue, int step, int page, int max);
private slots:
	void			vScroll (int value);
	void			hScroll (int value);
	void			clipboardClear ();

};

class QUWidget : public QFrame
{
	Q_OBJECT

public:
				QUWidget (QWidget* parent, const char* font);
				~QUWidget ();
	inline void		clearText () { twidget->uClear(); }
	void			appendText (const unsigned char* text, const int len);
	unsigned char*		getLine (const int pos, int *len);
	unsigned char*		getBuffer (int *len);
	void			refresh();
	UWidget::UStatus	setFontName (const char* font);
	UWidget::UStatus	setFontSize (const int fontSize);
	UWidget::UStatus	setFontWeight (const char* fontWeight);
	UWidget::UStatus	setFontSlant (const char* fontSlant);
	UWidget::UStatus	setFontSpacing (const char* fontSlant);
	UWidget::UStatus	setFontAddStyle (const char* addStyle);
	UWidget::UStatus	setFontAvgWidth (const char* avgWidth);
	UString::UStatus 	setConversion (const char *conversion);
	UWidget::UStatus 	setInput (const char *input);
	void			setTabSize (int newValue);
	void			setBackground (const char* color);
	void			setForeground (const char* color);
	void			setCursorColor (const char* color);
	void			setEditable (UWidget::UEditable editableIn);
	void			setMultiLine (UWidget::UMultiLine multiLineIn);
	void 			clearModified () 
				{twidget->uClearModified();}
	int 			isModified ()
				{return twidget->uIsModified();}
	const UCS2* const *	uGetTextBuffer();
	const int* 		uGetTextSizes();
	int			uGetTextSize();
	inline void		uSetXInput (UXInput*    xinputIn)
				{
					twidget->uSetXInput (xinputIn);
				}
	inline QUWidgetBase*	getWidget ()
				{
					return twidget;
				}
	inline void 		uSetCursor (UCursor::UType type, 
					const int all=0)
				{
					twidget->uSetCursor (type, all);
				}
	void			uAddEventHadler (UXInput *xinput, 
					UXInput::UInputEvent eventType);
	void			uRemoveEventHadler (UXInput *xinput, 
					UXInput::UInputEvent eventType);
	bool			uHandleXInputEvent (XEvent *e);

	int			uSetXInput (const char* name,
					const char*	imIn,
					const char*	inputStyleIn,
					const char*	textType,
					const char* 	uStringIn,
					const char*	font,
					double 		verisonIn=0.0);
	inline int		uFindUCS2 (const UCS2* text, const int len, int fromBegin=0)
				{
					return twidget->uFindUCS2 (text, len, fromBegin);
				}

	inline int		uReplaceUCS2 (const UCS2* text, const int len)
				{
					return twidget->uReplaceUCS2 (text,len);
				}
	inline int		uGoTo (int lineNo)
				{
					return twidget->uGoTo(lineNo);
				}
					
protected:
	virtual void		resizeEvent (QResizeEvent * event);
private:
	enum XKinputType 	{ KinputType, NoneType };
	UXInput*		xInput;
	UFontX11*		xInputFont;
	UFontMap*		xInputFontMap;
	AString			xInputFontMapName;
	XKinputType		xInputType;

	QColor			background;
	QColor			foreground;
	QColor			cursorColor;
	QUWidgetBase*		twidget;
	QScrollBar*		hscrollBar;
	QScrollBar*		vscrollBar;
	UWidget::UMultiLine	multiLine;

private slots:
	void			setVScroll (int value, int step, int page, int max);
	void			setHScroll (int value, int step, int page, int max);
};

QUWidget* QGetUWidgetItem (int index);

#endif /* __quwidget_h__ */
